;;; mn8-script.el --- mn8-script editing commands for XEmacs

;; Author: Antal Attila <atech@nolimits.ro>
;; Version: 1.1 
;; Maintainer: SC noLimits Technologies SRL
;; Keywords: languages, unix

;;; Commentary:

;; Major mode for editing mn8 scripts. 
;; - sintax highlightning
;; - mn8 menu for mn8 mode
;; - run the current mn8 scrit
;; - run mn8 command


;; TODO
;;
;; - tab aligning
;; - menu on mouse button



(defgroup mn8 nil
  "mn8 programming utilities"
  :group 'languages
  :prefix "mn8-")


(defgroup mn8-options nil
  "mn8 font faces"
  :group 'mn8
  :prefix "mn8-options-")
      

(defgroup mn8-font-faces nil
  "mn8 font faces"
  :group 'mn8
  :prefix "mn8-font-faces-")


(defcustom mn8-indentation 4
  "The width for further indentation in mn8-Script mode."
  :type 'integer
  :group 'mn8-options)


(defcustom java-home "/usr/java/jdk/"
  "Java home directory"
  :type 'string
  :group 'mn8-options)


(defcustom mn8-home "/devel/spacemapper/mn8/"
  "mn8 home directory"
  :type 'string
  :group 'mn8-options)


(defcustom mn8-command-interpreter "mn8"
  "mn8 interpreter command to run the mn8-script."
  :type 'string
  :group 'mn8-options)


(defcustom mn8-command-shell "/bin/bash"
  "mn8 interpreter command to run the mn8-script."
  :type 'string
  :group 'mn8-options)


(defface mn8-font-lock-concept-name-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "black" :bold t))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight Concept names"
   :group 'mn8-font-faces)


(defface mn8-font-lock-concept-def-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "red" :bold t))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight Concept names"
   :group 'mn8-font-faces)


(defface mn8-font-lock-keyword-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "red2"))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight common keywords"
   :group 'mn8-font-faces)


(defface mn8-font-lock-special-keyword-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "cyan4"))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight special keywords"
   :group 'mn8-font-faces)


(defface mn8-font-lock-logical-keyword-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "blue" :bold t))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight logical keywords"
   :group 'mn8-font-faces)


(defface mn8-font-lock-number-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "red4"))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight numbers"
   :group 'mn8-font-faces)


(defface mn8-font-lock-method-name-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "blue3" ))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight method names"
   :group 'mn8-font-faces)


(defface mn8-font-lock-variable-type-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "green4" :bold t))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight the type of variable"
   :group 'mn8-font-faces)


(defface mn8-font-lock-subconcept-name-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "dim grey" :bold t))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight the subconcept names"
   :group 'mn8-font-faces)


(defface mn8-font-lock-comment-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "blue4"))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight comments"
   :group 'mn8-font-faces)


(defface mn8-font-lock-variable-name-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "magenta1"))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight variable names"
   :group 'mn8-font-faces)


(defface mn8-font-lock-elementval-name-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "magenta4"))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight variable names"
   :group 'mn8-font-faces)


(defface mn8-font-lock-elementref-name-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "magenta4" :underline t))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight variable names"
   :group 'mn8-font-faces)


(defface mn8-font-lock-attribute-name-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "#8000ff"))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight variable names"
   :group 'mn8-font-faces)


(setq font-lock-face-list (append font-lock-face-list
                                '(mn8-font-lock-concept-name-face
                                  mn8-font-lock-concept-def-face
				  mn8-font-lock-keyword-face
				  mn8-font-lock-special-keyword-face
				  mn8-font-lock-logical-keyword-face
				  mn8-font-lock-number-face
				  mn8-font-lock-method-name-face
				  mn8-font-lock-variable-type-face
				  mn8-font-lock-subconcept-name-face
				  mn8-font-lock-comment-face
				  mn8-font-lock-variable-name-face
				  )))


(let*
((concept-keywords (concat "define\\|extends"))
(common-keywords
   (concat "typeof\\|do\\|label\\|exit\\|if\\|then\\|else\\|print\\|"
           "elif\\|while\\|each\\|in\\|from\\|to\\|return\\|repeat\\|"
	   "find\\|where\\|break\\|by\\|switch\\|options"))
(special-keywords (concat "first\\|last\\|ascending\\|descending"))
(logical-keywords (concat "not\\|or\\|and\\|true\\|false"))
(variable-type "static"))
  
  
(setq mn8-font-lock-keywords
  (eval-when-compile
    (list
      '("#\\([^\n]*\\)" 0 mn8-font-lock-comment-face t t)
      '("[/]\\([A-Za-z_]*\\)[/]" 1 mn8-font-lock-elementval-name-face)
      '("[$]\\([A-Za-z_]*\\)" 1 mn8-font-lock-variable-name-face)
      '("[/]\\([A-Za-z_]*\\)" 1 mn8-font-lock-elementref-name-face)
      '("[@]\\([A-Za-z_]*\\)" 1 mn8-font-lock-attribute-name-face)
      '("\\.\\([A-Za-z_]*\\)" 1 mn8-font-lock-method-name-face)
      '( "[ \t]+:[ \t]*\\([A-Za-z0-9_]*\\)" 1 mn8-font-lock-method-name-face)
      '("[A-Z]+[A-Za-z0-9_]*:\\([A-Za-z0-9_]*\\)" 1 mn8-font-lock-subconcept-name-face)
      '("\\([A-Z]+[A-Za-z0-9_]*\\)[\\.]?" 1 mn8-font-lock-concept-name-face)
      '("\\(this\\)[\\./@]?" 1 mn8-font-lock-keyword-face)
      '("\\(nil\\)[\\./@]?" 1 mn8-font-lock-concept-name-face)
      '("\\([0-9]+\\)" 1 mn8-font-lock-number-face)

      (cons (concat "\\<\\(" concept-keywords "\\)\\>")
            'mn8-font-lock-concept-def-face)
      (cons (concat "\\<\\(" variable-type "\\)\\>")
            'mn8-font-lock-variable-type-face)
      (cons (concat "\\<\\(" common-keywords "\\)\\>") 
            'mn8-font-lock-keyword-face) 
      (cons (concat "\\<\\(" special-keywords "\\)\\>")
            'mn8-font-lock-special-keyword-face)
      (cons (concat "\\<\\(" logical-keywords "\\)\\>")
            'mn8-font-lock-logical-keyword-face)
      ))))


(defun mn8-concept ()
  (interactive)
  (insert "\ndefine Concept label \"label\" extends Concept [\n\n]"))


(defun mn8-main ()
  (interactive)
  (insert "\n\tstatic : main ($args typeof Series) [\n\t\n\t]"))


(defun mn8-method ()
  (interactive)
  (insert "\n\t: method () [\n\t\n\t]"))


(defun mn8-smethod ()
  (interactive)
  (insert "\n\tstatic : method () [\n\t\n\t]"))


(defun mn8-all ()
  (interactive)
  (insert "\ndefine Concept label \"label\"  extends Concept [\n\n\t: create () [\n\t\n\t]\n\n\tstatic : main ($args typeof Series) [\n\t\n\t]\n\n\n]"))


(defvar mn8-set-command (if (eq system-type 'linux) "export" "set"))

(defun mn8-run ()
  (interactive)
  (require 'compile)
  (setq shell-file-name mn8-command-shell)
  (save-some-buffers (not compilation-ask-about-save))
  (compile-internal (concat mn8-command-interpreter " "
			    (if mn8-run-debug "-d ")
			    (if mn8-run-verbose "-v ")
			    (if mn8-run-cline
			        (concat "-c \"" mn8-run-cline "\"")
			        buffer-file-name ))
                      "No more errors." "Interpretation"
		      nil nil)
  (setq mn8-run-cline   nil
        x-get-selection nil))


(defun mn8-runc ()
  (interactive)
  (if (condition-case err (x-get-selection)  (error nil))
    (setq mn8-run-cline (x-get-selection))
    (setq mn8-run-cline (read-string "Enter the mn8 command: ")) )
  (mn8-run))


(defun mn8-show-options ()
  "Show the mn8 Options panel."
  (interactive)
  (customize-apropos "mn8-options" 'groups))


(defun mn8-show-font-faces ()
  "Show the mn8 font faces  panel."
  (interactive)
  (customize-apropos "mn8-font-faces" 'groups))


(defvar mn8-mode-map
  (let ((map  (make-keymap))
        (menu-map (make-sparse-keymap "Mn8")))
   (define-key map "\C-c\C-c" 'mn8-concept)
   (define-key map "\C-c\C-a" 'mn8-main)
   (define-key map "\C-c\C-m" 'mn8-method)
   (define-key map "\C-c\C-s" 'mn8-smethod)
   (define-key map "\C-c\C-f" 'mn8-all)
   (define-key map "\C-cx" 'mn8-run )
   (define-key map "\C-cc" 'mn8-runc )
   (define-key map "\t" 'mn8-indent-line)
   (substitute-key-definition 'newline-and-indent 'mn8-newline-and-indent
               map (current-global-map))
   map)
   "keymap for mn8 mode")


(defvar mn8-run-cline nil)
(defvar mn8-run-debug nil)
(defun mn8-toggle-run-debug()
   "Toggles debugg information"
   (interactive)
   (setq mn8-run-debug (not mn8-run-debug)))


(defvar mn8-run-verbose nil)
(defun mn8-toggle-run-verbose()
   "Toggles verbose information"
   (interactive)
   (setq mn8-run-verbose (not mn8-run-verbose)))


(defvar mn8-mode-menu
  (list "mn8"
        ["Options..."    mn8-show-options    t]
	["Font Faces..." mn8-show-font-faces t]
	"-"
        ["Run Script"  mn8-run     t]
	["Run Command" mn8-runc    t]
        (vector "Debug"
	        'mn8-toggle-run-debug
		:style 'toggle
		:selected 'mn8-run-debug)
	 (vector "Verbose"
                'mn8-toggle-run-verbose
                :style 'toggle
                :selected 'mn8-run-verbose)
        "-"
        (list "Insert"
	      ["Concept"       mn8-concept t]
	      ["Main method"   mn8-main    t]
	      ["Method"        mn8-method  t]
	      ["Static method" mn8-smethod t]
	      ["Full"          mn8-all     t]
	)
  )
  "Menu for mn8 mode")


(defun mn8-mode ()
  "mn8-mode"
  (interactive)
  (kill-all-local-variables)
  (use-local-map mn8-mode-map)
  (add-menu nil "mn8" (cdr mn8-mode-menu))
  (make-local-variable 'indent-line-function)                                   
  (make-local-variable 'indent-region-function)
  (setq major-mode 'mn8-mode
        mode-name  "mn8"
        indent-line-function 'mn8-indent-line	
	indent-region-function (lambda (b e)
	       (save-excursion (goto-char b)
	       (skip-syntax-backward "-")
	       (setq b (point)) (goto-char e)
	       (skip-syntax-backward "-")
	       (indent-rigidly b (point) mn8-indentation)))
	interpreter-mode-alist 
           (cons '("mn8" . mn8-mode) interpreter-mode-alist)
        font-lock-defaults
           '(mn8-font-lock-keywords nil nil
            ((?/ . "w") (?~ . "w") (?. . "w") (?- . "w") (?_ . "w")) )
  )
  (run-hooks 'mn8-mode-hook))


(defun mn8-indent-line ()
  "Indent as far as preceding non-empty line, then by steps of
  `mn8-indentation'. Lines containing only comments are considered empty."
  (interactive)
  (let ((previous (save-excursion
                    (while (and (progn (beginning-of-line)
                                       (not (bobp)))
 	                        (not (eq (point-min) (point-at-bol)))
			        (progn (forward-line -1)
				       (back-to-indentation)
				       (or (eolp)
				           (eq (following-char) ?#)))))
		    (current-column)))
	current)
  (save-excursion
    (indent-to (if (eq this-command 'mn8-newline-and-indent)
                    previous
		    (if (< (current-column)
		           (setq current (progn (back-to-indentation)
			                        (current-column))))
			(if (eolp) previous 0)
		    (delete-region (point)
		                   (progn (beginning-of-line) (point)))
	            (if (eolp)
		        (max previous (* (1+ (/ current mn8-indentation))
			                 mn8-indentation))
			              (* (1+ (/ current mn8-indentation))
				         mn8-indentation))))))
  (if (< (current-column) (current-indentation))
   	                  (skip-chars-forward " \t"))))


(defun mn8-quoted-p ()
  "Is point preceded by an odd number of backslashes?"
  (eq -1 (% (save-excursion (skip-chars-backward "\\\\")) 2)))


(defun mn8-newline-and-indent ()
  "Strip unquoted whitespace, insert newline, and indent like current line."    
  (interactive "*")
  (indent-to (prog1 (current-indentation)
                    (delete-region (point)
	               (progn (or (zerop (skip-chars-backward " \t"))
		                         (if (mn8-quoted-p)
					     (forward-char)))
		              (point)))
		    (newline))))

(provide 'mn8-script)
