;;; mn8-script.el --- mn8-script editing commands for XEmacs

;; Author: Antal Attila <atech@nolimits.ro>
;; Version: 1.0 (experimental fase)
;; Maintainer: SC noLimits Technologies SRL
;; Keywords: languages, unix

;;; Commentary:

;; Major mode for editing mn8 scripts. 
;; Now working only the sintax highlightning



(defcustom mn8-mode-hook nil
  "*Hook run by `mn8-mode'."
  :type 'hook 
  :group 'mn8)


(defgroup mn8 nil
  "mn8 programming utilities"
  :group 'unix
  :group 'languages)


(defgroup mn8-script nil
  "mn8 script mode"
  :group 'mn8
  :prefix "mn8-")


(defcustom mn8-indentation 4
  "The width for further indentation in mn8-Script mode."
  :type 'integer
  :group 'mn8-script)


(defface mn8-font-lock-concept-name-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "black" :bold t))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight Concept names"
   :group 'font-lock-faces)

(defface mn8-font-lock-concept-def-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "red" :bold t))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight Concept names"
   :group 'font-lock-faces)
   
(defface mn8-font-lock-keyword-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "red2"))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight common keywords"
   :group 'font-lock-faces)


(defface mn8-font-lock-special-keyword-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "cyan4"))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight special keywords"
   :group 'font-lock-faces)


(defface mn8-font-lock-logical-keyword-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "blue"))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight logical keywords"
   :group 'font-lock-faces)

(defface mn8-font-lock-number-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "red4"))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight numbers"
   :group 'font-lock-faces)


(defface mn8-font-lock-method-name-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "blue3" ))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight method names"
   :group 'font-lock-faces)


(defface mn8-font-lock-variable-type-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "green4" :bold t))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight the type of variable"
   :group 'font-lock-faces)


(defface mn8-font-lock-subconcept-name-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "dim grey" :bold t))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight the subconcept names"
   :group 'font-lock-faces)


(defface mn8-font-lock-comment-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "blue4"))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight comments"
   :group 'font-lock-faces)


(defface mn8-font-lock-variable-name-face
  '((((class color) (background dark)) (:foreground "cyan"))
   (((class color) (background light)) (:foreground "magenta3"))
   (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
   (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
   (t (:bold t)))
   "Font Lock mode face used to highlight variable names"
   :group 'font-lock-faces)


(setq font-lock-face-list (append font-lock-face-list
                                '(mn8-font-lock-concept-name-face
                                  mn8-font-lock-concept-def-face
				  mn8-font-lock-keyword-face
				  mn8-font-lock-special-keyword-face
				  mn8-font-lock-logical-keyword-face
				  mn8-font-lock-number-face
				  mn8-font-lock-method-name-face
				  mn8-font-lock-variable-type-face
				  mn8-font-lock-subconcept-name-face
				  mn8-font-lock-comment-face
				  mn8-font-lock-variable-name-face
				  )))


(let*
((concept-keywords (concat "define\\|extends"))
(common-keywords
   (concat "typeof\\|do\\|label\\|exit\\|if\\|then\\|else\\|print\\|"
           "elif\\|while\\|each\\|in\\|from\\|to\\|return\\|repeat\\|"
	   "find\\|first\\|last\\|where\\|break\\|by\\|switch\\|options"))
(special-keywords (concat "nil\\|true\\|false"))
(logical-keywords (concat "not\\|or\\|and"))
(variable-type "static")
)
  
(setq mn8-font-lock-keywords
  (eval-when-compile
    (list
      '("#\\([^\n]*\\)" 0 mn8-font-lock-comment-face t t)
      '("[$@/]\\([A-Za-z_]*\\)" 1 mn8-font-lock-variable-name-face)
      '("\\.\\([A-Za-z_]*\\)" 1 mn8-font-lock-method-name-face)
      '( "[ \t]+:[ \t]*\\([A-Za-z0-9_]*\\)" 1 mn8-font-lock-method-name-face)
      '("[A-Z]+[A-Za-z0-9_]*:\\([A-Za-z0-9_]*\\)" 1 mn8-font-lock-subconcept-name-face)
      '("\\([A-Z]+[A-Za-z0-9_]*\\)[\\.]?" 1 mn8-font-lock-concept-name-face)
      '("\\(this\\)[\\./@]?" 1 mn8-font-lock-keyword-face)
      '("\\([0-9]+\\)" 1 mn8-font-lock-number-face)

      (cons (concat "\\<\\(" concept-keywords "\\)\\>")
            'mn8-font-lock-concept-def-face)
      (cons (concat "\\<\\(" variable-type "\\)\\>")
            'mn8-font-lock-variable-type-face)
      (cons (concat "\\<\\(" common-keywords "\\)\\>") 
            'mn8-font-lock-keyword-face) 
      (cons (concat "\\<\\(" special-keywords "\\)\\>")
            'mn8-font-lock-special-keyword-face)
      (cons (concat "\\<\\(" logical-keywords "\\)\\>")
            'mn8-font-lock-logical-keyword-face)
      )))
)


(defun mn8-concept ()
  (interactive)
  (insert "define Concept label \"label\" extends Concept [\n\n]")
)

(defun mn8-main ()
  (interactive)
  (insert "\tstatic : main ($args typeof Series) [\n\t\n\t]")
)

(defun mn8-method ()
  (interactive)
  (insert "\t: method () [\n\t\n\t]")
)

(defun mn8-smethod ()
  (interactive)
  (insert "\tstatic : method () [\n\t\n\t]")
)

(defun mn8-all ()
  (interactive)
  (insert "define Concept label \"label\"  extends Concept [\n\n\t: create () [\n\t\n\t]\n\n\tstatic : main ($args typeof Series) [\n\t\n\t]\n\n\n]")
)

(defvar mn8-mode-map nil "") 

(if mn8-mode-map ()
   (setq mn8-mode-map (make-sparse-keymap))
   (define-key mn8-mode-map "\C-c\C-c" 'mn8-concept)
   (define-key mn8-mode-map "\C-c\C-a" 'mn8-main)
   (define-key mn8-mode-map "\C-c\C-m" 'mn8-method)
   (define-key mn8-mode-map "\C-c\C-s" 'mn8-smethod)
   (define-key mn8-mode-map "\C-c\C-f" 'mn8-all)
)


(defun mn8-mode ()
  "mn8-mode"
  (interactive)
  (use-local-map mn8-mode-map)
  (setq major-mode 'mn8-mode
        mode-name  "mn8-script"
        font-lock-defaults
           '(mn8-font-lock-keywords nil nil
            ((?/ . "w") (?~ . "w") (?. . "w") (?- . "w") (?_ . "w")) )
    )	     
  (run-hooks 'mn8-mode-hook))


(provide 'mn8-script)
